// IOCTLS.H -- IOCTL code definitions for WSUIO48_96XP driver
// Copyright (c) WinSystems.  All rights reserved.

#ifndef IOCTLS_H
    #define IOCTLS_H

#ifndef CTL_CODE
    #pragma message("CTL_CODE undefined. Include winioctl.h or wdm.h")
#endif

// Definition of the maximum number of UIO chips that can be contained in the device
#define MAX_DEVICES          2

// Maximum number of IO ports
#define MAX_IO_PORTS         6

// Maximum number of IO points per device
#define MAX_IO_POINTS        48

// Maximum number of bits in a byte
#define MAX_BITS_BYTE        8

// Maximum number of interrupt ports
#define MAX_INT_PORTS        3

// Maximum number of interrupt IO points per device
#define MAX_INT_POINTS       24

typedef unsigned char BYTE;
typedef unsigned char* PBYTE;
typedef unsigned long DWORD;
typedef unsigned long* PDWORD;

// Structure to use when writing a value to an I/O point or port using IOCTL_WIO_WRITE_BIT and IOCTL_WIO_WRITE_BYTE
typedef struct _UIOWriteValue
{
    union
    {
        DWORD  dwBitNumber;                                // Bit number to access - valid range: 1-48 (UIO-48) or 1-96 (UIO-96)
        DWORD  dwPortNumber;                               // Port number to access - valid range: 1-6 (UIO-48) or 1-12 (UIO-96)
    };

    BYTE   byValue;                                        // Value to write, 1 or 0

} UIO_WRITE_VALUE, *PUIO_WRITE_VALUE;

// Structure to use when configuring interrupts using IOCTL_WIO_ENAB_INT
typedef struct _UIOConfigInterrupt
{
    union
    {
        DWORD  dwBitNumber;                                // Bit number to access - valid range: 1-48 (UIO-48) or 1-96 (UIO-96)
        DWORD  dwPortNumber;                               // Port number to access - valid range: 1-6 (UIO-48) or 1-12 (UIO-96)
    };

    BYTE   byPolarity;                                     // Interrupt polarity - 1 = rising-edge, 0 = falling-edge

} UIO_CONFIG_INTERRUPT, *PUIO_CONFIG_INTERRUPT;

// Define the type of device. This parameter can be no bigger than a WORD value. The values used by Microsoft are in the range of
// 0-32767 and the values between 32768-65535 are reserved for use by OEMs and IHVs
#define WIO_DEV_TYPE         32768

//---------------------------------------------------------------------------------------------------------------------------------
// IOCTL_WIO_SET_IO_MASK
//
// Device IO control routine to set the I/O mask definition
//
// dwCode          IOCTL_WIO_SET_IO_MASK
// cbin            [IN] Size of input data buffer. Length of the I/O mask definition - 6 bytes (UIO-48) or 12 bytes (UIO-96)
// cbout           Ignored
// pCopyBuffer     [IN] Pointer to an unsigned character array holding the I/O mask definition. A 0 represents the I/O point as
//                 an input, 1 represents the I/O point as an output. There must be 6 bytes (UIO-48) or 12 bytes (UIO-96)
//
//                 Byte#    0 = bits 7 - 0
//                          1 = bits 15 - 8
//                          2 = bits 23 - 16
//                          3 = bits 31 - 24
//                          4 = bits 39 - 32
//                          5 = bits 47 - 40
//
//                 Sample mask definition:
//                          Bit#     76543210
//                          Mask     10101010
//                                   ||||||||_ Input
//                                   |||||||__ Output
//                                   ||||||___ Input
//                                   |||||____ Output
//                                   ||||_____ Input
//                                   |||______ Output
//                                   ||_______ Input
//                                   |________ Output
//
// info            [OUT] Number of output bytes put into the copy buffer
//---------------------------------------------------------------------------------------------------------------------------------

#define IOCTL_WIO_SET_IO_MASK   CTL_CODE(WIO_DEV_TYPE, 0x800, METHOD_BUFFERED, FILE_ANY_ACCESS)

//---------------------------------------------------------------------------------------------------------------------------------
// IOCTL_WIO_GET_IO_MASK
//
// Device IO control routine to get the I/O mask definition
//
// dwCode          IOCTL_WIO_GET_IO_MASK
// cbin            Ignored
// cbout           [IN] Size of output data buffer. Length of the I/O mask definition - 6 bytes (UIO-48) or 12 bytes (UIO-96)
// pCopyBuffer     [OUT] Pointer to an unsigned character array to receive the I/O mask definition. A 0 represents the I/O point
//                 as an input, 1 represents the I/O point as an output. There must be 6 bytes (UIO-48) or 12 bytes (UIO-96)
//
//                 Byte#    0 = bits 7 - 0
//                          1 = bits 15 - 8
//                          2 = bits 23 - 16
//                          3 = bits 31 - 24
//                          4 = bits 39 - 32
//                          5 = bits 47 - 40
//
//                 Sample mask definition:
//                          Bit#     76543210
//                          Mask     10101010
//                                   ||||||||_ Input
//                                   |||||||__ Output
//                                   ||||||___ Input
//                                   |||||____ Output
//                                   ||||_____ Input
//                                   |||______ Output
//                                   ||_______ Input
//                                   |________ Output
//
// info [OUT] Number of output bytes put into the copy buffer
//---------------------------------------------------------------------------------------------------------------------------------

#define IOCTL_WIO_GET_IO_MASK   CTL_CODE(WIO_DEV_TYPE, 0x801, METHOD_BUFFERED, FILE_ANY_ACCESS)

//---------------------------------------------------------------------------------------------------------------------------------
// IOCTL_WIO_CLEAR_IO_MASK
//
// Device IO control routine to clear the I/O mask definition
//
// code IOCTL_WIO_CLEAR_IO_MASK
// cbin Ignored
// cbout Ignored
// pCopyBuffer Ignored
// info [OUT] Number of output bytes put into the copy buffer
//---------------------------------------------------------------------------------------------------------------------------------

#define IOCTL_WIO_CLEAR_IO_MASK   CTL_CODE(WIO_DEV_TYPE, 0x802, METHOD_BUFFERED, FILE_ANY_ACCESS)

//---------------------------------------------------------------------------------------------------------------------------------
// IOCTL_WIO_READ_BIT
//
// Device IO control routine to get the state of the I/O pin specified
//
// code            IOCTL_WIO_READ_BIT
// cbin            [IN] Size of input data buffer, should be sizeof(DWORD)
// cbout           [IN] Size of output data buffer, should be sizeof(BYTE)
// pCopyBuffer     [IN/OUT]
//                 [IN] Pointer a DWORD containing the bit number to read - valid range: 1-48 (UIO-48) or 1-96 (UIO-96)
//                 [OUT] Pointer to a BYTE containing the state of the I/O pin. A 0 indicates pin is in High Impedance state, 
//                       a 1 indicates pin is in Low Impedance state
// info            [OUT] Number of output bytes put into the copy buffer
//---------------------------------------------------------------------------------------------------------------------------------

#define IOCTL_WIO_READ_BIT   CTL_CODE(WIO_DEV_TYPE, 0x803, METHOD_BUFFERED, FILE_ANY_ACCESS)

//---------------------------------------------------------------------------------------------------------------------------------
// IOCTL_WIO_WRITE_BIT
//
// Device IO control routine to set the state of the I/O pin specified
//
// code            IOCTL_WIO_WRITE_BIT
// cbin            [IN] Size of input data buffer,  should be sizeof(UIO_WRITE_VALUE)
// cbout           Ignored
// pCopyBuffer     [IN] Pointer a UIO_WRITE_VALUE structure
//                           UIO_WRITE_VALUE structure
//                           dwBitNumber   The bit number to write value to - valid range: 1-48 (UIO-48) or 1-96 (UIO-96)
//                           byValue       Value to write to bit. A 0 causes the pin to go to a High Impedance state, a 1 causes
//                                         the pin to go to a Low Impedance state
// info            [OUT] Number of output bytes put into the copy buffer
//---------------------------------------------------------------------------------------------------------------------------------

#define IOCTL_WIO_WRITE_BIT   CTL_CODE(WIO_DEV_TYPE, 0x804, METHOD_BUFFERED, FILE_ANY_ACCESS)

//---------------------------------------------------------------------------------------------------------------------------------
// IOCTL_WIO_SET_BIT
//
// Device IO control routine to set the state of the I/O pin specified to a Low Impedance state
//
// code            IOCTL_WIO_SET_BIT
// cbin            [IN] Size of input data buffer, should be sizeof(DWORD)
// cbout           Ignored
// pCopyBuffer     [IN] Pointer a DWORD containing the bit number to set - valid range: 1-48 (UIO-48) or 1-96 (UIO-96)
// info            [OUT] Number of output bytes put into the copy buffer
//---------------------------------------------------------------------------------------------------------------------------------

#define IOCTL_WIO_SET_BIT   CTL_CODE(WIO_DEV_TYPE, 0x805, METHOD_BUFFERED, FILE_ANY_ACCESS)

//---------------------------------------------------------------------------------------------------------------------------------
// IOCTL_WIO_CLR_BIT
//
// Device IO control routine to set the state of the I/O pin specified to a High Impedance state
//
// code            IOCTL_WIO_CLR_BIT
// cbin            [IN] Size of input data buffer, should be sizeof(DWORD)
// cbout           Ignored
// pCopyBuffer     [IN] Pointer a DWORD containing the bit number to clear - valid range: 1-48 (UIO-48) or 1-96 (UIO-96)
// info            [OUT] Number of output bytes put into the copy buffer
//---------------------------------------------------------------------------------------------------------------------------------

#define IOCTL_WIO_CLR_BIT   CTL_CODE(WIO_DEV_TYPE, 0x806, METHOD_BUFFERED, FILE_ANY_ACCESS)

//---------------------------------------------------------------------------------------------------------------------------------
// IOCTL_WIO_READ_BYTE
//
// Device IO control routine to get the state of all I/O pins in the specified port
//
// code            IOCTL_WIO_READ_BYTE
// cbin            [IN] Size of input data buffer, should be sizeof(DWORD)
// cbout           [IN] Size of output data buffer, should be sizeof(BYTE)
// pCopyBuffer     [IN/OUT]
//                 [IN] Pointer a DWORD containing the port number to read - valid range: 1-6 (UIO-48) or 1-12 (UIO-96)
//                 [OUT] Pointer to a BYTE containing the state of the I/O port
// info            [OUT] Number of output bytes put into the copy buffer
//---------------------------------------------------------------------------------------------------------------------------------

#define IOCTL_WIO_READ_BYTE   CTL_CODE(WIO_DEV_TYPE, 0x807, METHOD_BUFFERED, FILE_ANY_ACCESS)

//---------------------------------------------------------------------------------------------------------------------------------
// IOCTL_WIO_WRITE_BYTE
//
// Device IO control routine to set the state of all I/O pins in the specified port
//
// code            IOCTL_WIO_WRITE_BYTE
// cbin            [IN] Size of input data buffer, should be sizeof(UIO_WRITE_VALUE)
// cbout           Ignored
// pCopyBuffer     [IN] Pointer a UIO_WRITE_VALUE structure
//                           UIO_WRITE_VALUE structure
//                           dwPortNumber   The port number to write value to - valid range: 1-6 (UIO-48) or 1-12 (UIO-96)
//                           byValue        Value to write to port
// info            [OUT] Number of output bytes put into the copy buffer
//---------------------------------------------------------------------------------------------------------------------------------

#define IOCTL_WIO_WRITE_BYTE   CTL_CODE(WIO_DEV_TYPE, 0x808, METHOD_BUFFERED, FILE_ANY_ACCESS)

//---------------------------------------------------------------------------------------------------------------------------------
// IOCTL_WIO_ENAB_INT
//
// Device IO control routine to enable an interrupt for the specified I/O pin at a specified polarity
//
// code            IOCTL_WIO_ENAB_INT
// cbin            [IN] Size of input data buffer, should be sizeof(UIO_CONFIG_INTERRUPT)
// cbout           Ignored
// pCopyBuffer     [IN] Pointer a UIO_CONFIG_INTERRUPT structure
//                           UIO_CONFIG_INTERRUPT structure
//                           dwBitNumber   The bit number to enable interrupts on - valid range: 1-24 (UIO-48) or 1-48 (UIO-96)
//                           byPolarity    The polarity of the interrupt - non-zero enables rising-edge interrupt, zero enables
//                                         falling-edge interrupt
// info            [OUT] Number of output bytes put into the copy buffer
//---------------------------------------------------------------------------------------------------------------------------------

#define IOCTL_WIO_ENAB_INT   CTL_CODE(WIO_DEV_TYPE, 0x809, METHOD_BUFFERED, FILE_ANY_ACCESS)

//---------------------------------------------------------------------------------------------------------------------------------
// IOCTL_WIO_DISAB_INT
//
// Device IO control routine to disable an interrupt for the specified I/O pin
//
// code            IOCTL_WIO_DISAB_INT
// cbin            [IN] Size of input data buffer, should be sizeof(DWORD)
// cbout           Ignored
// pCopyBuffer     [IN] Pointer a DWORD containing the bit number to disable interrupts on - valid range: 1-24 (UIO-48) or
//                      1-48 (UIO-96)
// info            [OUT] Number of output bytes put into the copy buffer
//---------------------------------------------------------------------------------------------------------------------------------

#define IOCTL_WIO_DISAB_INT   CTL_CODE(WIO_DEV_TYPE, 0x80A, METHOD_BUFFERED, FILE_ANY_ACCESS)

//---------------------------------------------------------------------------------------------------------------------------------
// IOCTL_WIO_WAIT_ON_INT
//
// Device IO control routine to wait for an interrupt for any bits that have been specified on prior calls to IOCTL_WIO_ENAB_INT
//
// code            IOCTL_WIO_WAIT_ON_INT
// cbin            Ignored
// cbout           [IN] Size of output data buffer - There must be 3 bytes (UIO-48) or 6 bytes (UIO-96)
// pCopyBuffer     [OUT] Pointer to an unsigned character array to receive the I/O bits that have triggered an interrupt. There
//                       must be 3 bytes (UIO-48) or 6 bytes (UIO-96).
//
//                       In a UIO-48 device there are 48 I/O points. The UIO-48 has the ability to monitor 24 I/O points for both
//                       rising and falling digital edge transitions and issue an interrupt. The first 24 I/O points are the
//                       points that can be configured to issue an interrupt upon the rising or falling digital edge transition.
//                       In a UIO-96 device there are 96 I/O points. The UIO-96 has the ability to monitor 48 I/O points for both
//                       rising and falling digital edge transitions and issue an interrupt. The UIO-96 device contains two
//                       WinSystems' WS16C48 ASICs. Therefore, the 48 I/O points that may be configured for interrupts span
//                       across the two chips. The 48 I/O points that may be configured for interrupts correspond to I/O bits 1-24
//                       and bits 49-72. Below is a chart that details the format of the buffer pointed to by pBufOut.
//
//                       Byte#       0 = Int. bits  8 -  1 I/O bits  8 -  1
//                                   1 = Int. bits 16 -  9 I/O bits 16 -  9
//                                   2 = Int. bits 24 - 17 I/O bits 24 - 17
//                                   3 = Int. bits 32 - 25 I/O bits 56 - 49
//                                   4 = Int. bits 40 - 33 I/O bits 64 - 57
//                                   5 = Int. bits 48 - 41 I/O bits 72 - 65
//
//                       Sample return value:
//                                   Bit#     76543210
//                                   Value    10101010
//                                            ||||||||_ No Interrupt
//                                            |||||||__ Interrupt
//                                            ||||||___ No Interrupt
//                                            |||||____ Interrupt
//                                            ||||_____ No Interrupt
//                                            |||______ Interrupt
//                                            ||_______ No Input
//                                            |________ Interrupt
//
// info            [OUT] Number of output bytes put into the copy buffer
//---------------------------------------------------------------------------------------------------------------------------------

#define IOCTL_WIO_WAIT_ON_INT   CTL_CODE(FILE_DEVICE_UNKNOWN, 0x80B, METHOD_BUFFERED, FILE_ANY_ACCESS)

//---------------------------------------------------------------------------------------------------------------------------------
// IOCTL_WIO_GET_NUM_IO_POINTS
//
// Device IO control routine to get the number of I/O points in the configuration
//
// code            IOCTL_WIO_GET_NUM_IO_POINTS
// cbin            Ignored
// cbout           [IN] Size of output data buffer, should be sizeof(DWORD)
// pCopyBuffer     [OUT] Pointer a DWORD to receive the number of I/O points in the configuration
// info            [OUT] Number of output bytes put into the copy buffer
//---------------------------------------------------------------------------------------------------------------------------------

#define IOCTL_WIO_GET_NUM_IO_POINTS   CTL_CODE(WIO_DEV_TYPE, 0x80C, METHOD_BUFFERED, FILE_ANY_ACCESS)

//---------------------------------------------------------------------------------------------------------------------------------
// IOCTL_WIO_INT_PERMITTED
//
// Device IO control routine to determine if interrupts are permitted in the driver
//
// code            IOCTL_WIO_INT_PERMITTED
// cbin            Ignored
// cbout           [IN] Size of output data buffer, should be sizeof(DWORD)
// pCopyBuffer     [OUT] Pointer a DWORD to receive a value of 1 if interrupts are permitted, 0 if interrupts are not permitted
// info            [OUT] Number of output bytes put into the copy buffer
//---------------------------------------------------------------------------------------------------------------------------------

#define IOCTL_WIO_INT_PERMITTED   CTL_CODE(WIO_DEV_TYPE, 0x80D, METHOD_BUFFERED, FILE_ANY_ACCESS)

//---------------------------------------------------------------------------------------------------------------------------------
// IOCTL_WIO_READ_DWORD
//
// Device IO control routine to get the state of all I/O pins in the specified four ports
//
// code            IOCTL_WIO_READ_DWORD
// cbin            [IN] Size of input data buffer, should be sizeof(DWORD)
// cbout           [IN] Size of output data buffer, should be sizeof(DWORD)
// pCopyBuffer     [IN/OUT]
//                 [IN] Pointer to a DWORD containing the port number to read - valid range: 1-3 (UIO-48G)
//                 [OUT] Pointer to a DWORD containing the state of the I/O port
// info            [OUT] Number of output bytes put into the copy buffer
//---------------------------------------------------------------------------------------------------------------------------------

#define IOCTL_WIO_READ_DWORD   CTL_CODE(WIO_DEV_TYPE, 0x80E, METHOD_BUFFERED, FILE_ANY_ACCESS)

//---------------------------------------------------------------------------------------------------------------------------------
// IOCTL_WIO_READ_ALL_PORTS
//
// Device IO control routine to get the state of all I/O pins in the specified four ports
//
// code            IOCTL_WIO_READ_DWORD
// cbin            Ignored
// cbout           [IN] Size of output data buffer, should be sizeof(DWORDLONG)
// pCopyBuffer     [IN/OUT]
//                 [OUT] Pointer to a DWORDLONG containing the state of all I/O ports
// info            [OUT] Number of output bytes put into the copy buffer
//---------------------------------------------------------------------------------------------------------------------------------

#define IOCTL_WIO_READ_ALL_PORTS   CTL_CODE(WIO_DEV_TYPE, 0x80F, METHOD_BUFFERED, FILE_ANY_ACCESS)

#endif
